==> lhcsmapi-1.2.19.tar.gz: lhcsmapi-1.2.19/lhcsmapi/analysis/RbQuery.py <==
import scipy as sp
import pandas as pd
from tqdm.notebook import tqdm

# lhc-sm-api
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.metadata.SignalMetadata import SignalMetadata

# Analysis modules
from lhcsmapi.analysis.qds.iqds.IqdsQuery import IqdsQuery
from lhcsmapi.analysis.qds.iqds.IqdsAnalysis import IqdsAnalysis
from lhcsmapi.analysis.pc.PowerConverterQuery import PowerConverterQuery
from lhcsmapi.analysis.pc.PowerConverterAnalysis import PowerConverterAnalysis
from lhcsmapi.analysis.leads.CurrentLeadsQuery import CurrentLeadsQuery
from lhcsmapi.analysis.qds.nqds.NqdsQuery import NqdsQuery
from lhcsmapi.analysis.nqps.NqpsQuery import NqpsQuery
from lhcsmapi.analysis.ee.EnergyExtractionQuery import EnergyExtractionQuery
from lhcsmapi.analysis.diode.DiodeLeadResistanceQuery import DiodeLeadResistanceQuery
from lhcsmapi.analysis.qh.QuenchHeaterQuery import QuenchHeaterQuery


class RbQuery(object):

    def __init__(self, circuit_name, circuit_type, timestamp_fgc, author):
        self.circuit_name = circuit_name
        self.circuit_type = circuit_type
        self.timestamp_fgc = timestamp_fgc
        self.author = author

    def get_power_converter_main_current(self, verbose=True):
        if verbose:
            print('Querying PM for Power Converter main current, I_MEAS.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        i_meas_df, i_meas_ref_df, timestamp_fgc_ref = PowerConverterAnalysis.analyze_fgc_current(metadata_fgc,
                                                                                                 self.timestamp_fgc,
                                                                                                 self.circuit_name)
        if i_meas_df['I_MEAS'].max() < 1000:
            raise Warning('Analysis for current below 1 kA may provide unexpected results!')

        return i_meas_df, i_meas_ref_df, timestamp_fgc_ref

    def get_pic_timestamp(self, ldb, verbose=True):
        # Find PIC timestamp
        if verbose:
            print('Querying CALS for PIC abort signal, ST_ABORT_PIC.')
        return PowerConverterQuery.get_st_abort_pic_timestamp(self.circuit_type, self.circuit_name,
                                                              self.timestamp_fgc, ldb)

    def get_power_converter_earth_current(self, timestamp_fgc_ref, verbose=True):
        if verbose:
            print('Querying PM for Power Converter earth current, IEARTH.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        return PowerConverterQuery.query_earth_current(metadata_fgc, self.timestamp_fgc, timestamp_fgc_ref)

    def get_power_converter_earth_current_pcnt(self, timestamp_fgc_ref, scaling, verbose=True):
        if verbose:
            print('Querying PM for Power Converter earth current, I_EARTH_PCNT.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        return PowerConverterQuery.query_earth_current_pcnt(metadata_fgc, self.timestamp_fgc, timestamp_fgc_ref,
                                                            scaling=scaling)

    def get_power_converter_reference_current(self, verbose=True):
        if verbose:
            print('Querying PM for Power Converter reference current, STATUS.I_REF.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        return PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, 'STATUS.I_REF', self.timestamp_fgc)

    def get_energy_extraction_voltage(self, verbose=True):
        if verbose:
            print('Querying PM for Energy Extraction voltage, U_DUMP_RES.')

        metadata_ee = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'EE',
                                                                        'PM')
        return EnergyExtractionQuery.query_u_dump_res(metadata_ee, self.timestamp_fgc)

    def get_energy_extraction_temperature_pm(self, verbose=True):
        if verbose:
            print('Querying PM for Energy Extraction temperature, T_RES_BODY.')

        metadata_ee = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'EE',
                                                                        'PM')
        T_RES_BODY = [
            SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'EE', 'PM')[
                'T_RES_BODY_{}'.format(i)] for i in range(1, 4)]
        return EnergyExtractionQuery.query_t_res_body_pm(metadata_ee, T_RES_BODY, self.timestamp_fgc)

    def get_energy_extraction_temperature_cals(self, ldb, verbose=True):
        if verbose:
            print('Querying CALS for Energy Extraction temperature, T_RES_BODY.')

        variable_name = ["T_RES_BODY_1", "T_RES_BODY_2", "T_RES_BODY_3"]
        return EnergyExtractionQuery.query_cals(self.circuit_type, self.circuit_name, self.timestamp_fgc,
                                                variable_name, ldb)

    def get_energy_extraction_status_temperature_cals(self, ldb, verbose=True):
        if verbose:
            print('Querying CALS for Energy Extraction temperature, ST_RES_OVERTEMP.')

        variable_name = "ST_RES_OVERTEMP"
        return EnergyExtractionQuery.query_cals(self.circuit_type, self.circuit_name, self.timestamp_fgc,
                                                variable_name, ldb)

    def get_nqps_voltage(self, system, signal, ldb, prefix='', verbose=True):
        if verbose:
            print('Querying CALS for {} voltage, {}.'.format(system, signal))

        return NqpsQuery.query_nqps_voltage(self.circuit_type, self.circuit_name, self.timestamp_fgc, system=system,
                                            signal=signal, prefix=prefix, ldb=ldb)

    def get_nqps_pm_source_timestamp_df(self, verbose=True):
        if verbose:
            print('Querying PM for QPS diode voltage, U_DIODE_RB.')

        metadata_diode = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name,
                                                                           'DIODE_{}'.format(self.circuit_type), 'PM')
        source_timestamp_nqps = PmDbRequest.find_events('*', metadata_diode['system'], metadata_diode['className'],
                                                        t_start=self.timestamp_fgc, duration=[(10, 's'), (400, 's')])

        print("Total nQPS PM events on {}: {}".format(Time.to_string(self.timestamp_fgc),
                                                      len(source_timestamp_nqps)))

        if len(source_timestamp_nqps) > 0:
            source_timestamp_nqps_df = pd.DataFrame(source_timestamp_nqps).drop_duplicates(subset=[0]).sort_values(
                by=[1]).reset_index(drop=True)
            source_timestamp_nqps_df.columns = ['nQPS', 'timestamp_nqps']
            source_timestamp_nqps_df['timestamp_nqps'] = source_timestamp_nqps_df['timestamp_nqps'].astype('Int64')
        else:
            source_timestamp_nqps_df = pd.DataFrame(columns=['nQPS', 'timestamp_nqps'])
        return source_timestamp_nqps_df

    def create_report_analysis(self, source_timestamp_nqps_df, i_meas_df, verbose=True):
        if verbose:
            print('Querying PM for QPS quench detection voltage, U_QS0.')

        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'QDS',
                                                                         'PM')
        event_source_timestamp_qds = PmDbRequest.find_events('*', metadata_qds['system'], metadata_qds['className'],
                                                             t_start=self.timestamp_fgc,
                                                             duration=[(1, 's'), (1000, 's')])
        # Take only the qds events in the analyzed circuit
        event_source_timestamp_qds_filtered = list(
            filter(lambda x: SignalMetadata.get_sector_names_from_components(self.circuit_type, 'MAGNET', x[0]) in
                             self.circuit_name, event_source_timestamp_qds))

        # Create report analysis template
        report_analysis = IqdsAnalysis.create_analysis_report(self.circuit_type, self.circuit_name, metadata_qds,
                                                              event_source_timestamp_qds_filtered, i_meas_df,
                                                              self.timestamp_fgc)
        report_analysis['doneBy'] = self.author
        # nQPS - append timestamps
        report_analysis = report_analysis.merge(source_timestamp_nqps_df[['timestamp_nqps', 'nQPS']],
                                                on='nQPS', how='left').sort_values(by='timeStampPM').reset_index(
                                                drop=True)

        return report_analysis

    def get_iqps_nqps_voltage_logic(self, report_analysis, ldb, verbose=True):
        if verbose:
            print(
                'Querying PM and CALS for QPS quench detection voltage and logic signals, U_QS0, U_1, U_2, U_DIODE_RB, ST_NQD0, ST_MAGNET_OK.')

        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'QDS',
                                                                         'PM')
        # ToDo: input: list of source_qds, list of source_nqs, list of timestamp_qds, list of timestamp_nqps, list of i_quench to avoid using report_analysis
        u_qds_dfs, u_nqps_dfs, st_nqps_dfs = [], [], []

        for index, row in tqdm(report_analysis.iterrows(), total=report_analysis.shape[0], desc='Querying PM'):
            source_qds, source_nqps, timestamp_qds, timestamp_nqps, i_quench = row[
                ['magnet', 'nQPS', 'timeStampPM', 'timestamp_nqps', 'qCurrent']]

            # Query
            # # iQPS
            u_qds_df = IqdsQuery.query_synchronize_pm_signal(metadata_qds,
                                                             ['U_QS0', 'U_1', 'U_2', 'ST_NQD0', 'ST_MAGNET_OK'],
                                                             timestamp_qds, source_qds)
            u_qds_df[['U_QS0', 'U_1', 'U_2']] = u_qds_df[['U_QS0', 'U_1', 'U_2']].apply(sp.signal.medfilt)
            u_qds_dfs.append(u_qds_df)
            # # nQPS
            u_nqps_df = NqdsQuery.query_nqps_pm_cals(self.circuit_type, self.circuit_name, source_nqps,
                                                     timestamp_nqps, timestamp_qds, ldb)
            u_nqps_dfs.append(u_nqps_df)

        return u_qds_dfs, u_nqps_dfs

    def get_qh_voltage_current(self, report_analysis, verbose=True):
        if verbose:
            print('Querying PM for QH voltage and current, U_HDS, I_HDS.')
        qh_sync_decay_dfs, qh_ref_sync_decay_dfs = [], []
        for index, row in tqdm(report_analysis.iterrows(), total=report_analysis.shape[0], desc='Querying PM'):
            timestamp_qds = row['timeStampPM'] + 1000000
            source_qds = row['magnet']

            # Acquire
            qh_sync_decay_df, qh_ref_sync_decay_df = QuenchHeaterQuery.get_actual_reference_discharges(
                self.circuit_type, self.circuit_name, source_qds, timestamp_qds)
            qh_sync_decay_dfs.append(qh_sync_decay_df)
            qh_ref_sync_decay_dfs.append(qh_ref_sync_decay_df)

        return qh_sync_decay_dfs, qh_ref_sync_decay_dfs

    def get_diode_leads_voltage_current(self, report_analysis, ldb, verbose=True):
        if verbose:
            print('Querying PM for diode lead resistance voltage and current, U_DIODE_RB, U_REF, IAB.I_A, I_MEAS.')

        iab_i_as, u_diode_as, u_diode_bs, u_refs = [], [], [], []
        i_circuit_syncs, u_diode_syncs = [], []
        for index, row in tqdm(report_analysis.iterrows(), total=report_analysis.shape[0], desc='Querying PM'):
            source_qds, timestamp_qds, i_quench = row[['magnet', 'timeStampPM', 'qCurrent']]

            if i_quench < 1000:
                print('The query skipped as the quench current is below 1 kA ({} A)'.format(i_quench))
            else:
                # Query
                # # PM only if a quench is within two seconds after an nqps timestamp
                timestamp_diode_fgc = DiodeLeadResistanceQuery.find_pc_pm_entries(self.circuit_type, self.circuit_name,
                                                                                  timestamp_qds)
                timestamp_diode_nqps = DiodeLeadResistanceQuery.find_diode_pm_entries(self.circuit_type,
                                                                                      self.circuit_name, source_qds,
                                                                                      timestamp_qds)
                if len(timestamp_diode_nqps) > 0:
                    is_quench_in_nqps = abs(timestamp_diode_nqps[0] - timestamp_qds) < 2e9
                else:
                    is_quench_in_nqps = False

                if is_quench_in_nqps:
                    # # # IAB.I_A
                    iab_i_as.append(DiodeLeadResistanceQuery.query_iab_i_a(self.circuit_type, self.circuit_name,
                                                                           timestamp_diode_fgc[0]))
                    # # # U_DIODE
                    u_diode_as.append(
                        DiodeLeadResistanceQuery.query_u_diode(self.circuit_type, self.circuit_name, source_qds,
                                                               timestamp_diode_nqps[0]))
                    u_diode_bs.append(
                        DiodeLeadResistanceQuery.query_u_diode(self.circuit_type, self.circuit_name, source_qds,
                                                               timestamp_diode_nqps[1]))
                    # # # U_REF
                    u_refs.append(DiodeLeadResistanceQuery.query_u_ref(self.circuit_type, self.circuit_name, source_qds,
                                                                       timestamp_diode_nqps[0]))

                # # CALS
                magnet_type, I_MEAS, U_DIODE = DiodeLeadResistanceQuery.get_magnet_type_i_meas_u_diode(source_qds,
                                                                                                       self.circuit_name)

                i_circuit_syncs.append(
                    DiodeLeadResistanceQuery.query_cals_signal(I_MEAS, timestamp_qds, magnet_type, ldb=ldb))
                u_diode_syncs.append(
                    DiodeLeadResistanceQuery.query_cals_signal(U_DIODE, timestamp_qds, magnet_type, ldb=ldb))

        return iab_i_as, u_diode_as, u_diode_bs, u_refs, i_circuit_syncs, u_diode_syncs

    def get_current_leads_voltage(self, leads_type, verbose=True):
        if verbose:
            print('Querying PM for {} voltage, U_RES, U_HTS.'.format(leads_type))

        return CurrentLeadsQuery.query_u_leads(self.circuit_type, self.circuit_name, self.timestamp_fgc,
                                               leads_type=leads_type)

==> lhcsmapi-1.2.49.tar.gz: lhcsmapi-1.2.49/lhcsmapi/analysis/RbQuery.py <==
import scipy as sp
import pandas as pd
from tqdm.notebook import tqdm

# lhc-sm-api
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.metadata.SignalMetadata import SignalMetadata

# Analysis modules
from lhcsmapi.analysis.qds.iqds.IqdsQuery import IqdsQuery
from lhcsmapi.analysis.qds.iqds.IqdsAnalysis import IqdsAnalysis
from lhcsmapi.analysis.pc.PowerConverterQuery import PowerConverterQuery
from lhcsmapi.analysis.pc.PowerConverterAnalysis import PowerConverterAnalysis
from lhcsmapi.analysis.leads.CurrentLeadsQuery import CurrentLeadsQuery
from lhcsmapi.analysis.qds.nqds.NqdsQuery import NqdsQuery
from lhcsmapi.analysis.nqps.NqpsQuery import NqpsQuery
from lhcsmapi.analysis.ee.EnergyExtractionQuery import EnergyExtractionQuery
from lhcsmapi.analysis.diode.DiodeLeadResistanceQuery import DiodeLeadResistanceQuery
from lhcsmapi.analysis.qh.QuenchHeaterQuery import QuenchHeaterQuery


class RbQuery(object):

    def __init__(self, circuit_name, circuit_type, timestamp_fgc, author):
        self.circuit_name = circuit_name
        self.circuit_type = circuit_type
        self.timestamp_fgc = timestamp_fgc
        self.author = author

    def get_power_converter_main_current(self, verbose=True):
        if verbose:
            print('Querying PM for Power Converter main current, I_MEAS.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        i_meas_df, i_meas_ref_df, timestamp_fgc_ref = PowerConverterAnalysis.analyze_fgc_current(metadata_fgc,
                                                                                                 self.timestamp_fgc,
                                                                                                 self.circuit_name)

        return i_meas_df, i_meas_ref_df, timestamp_fgc_ref

    def get_pic_timestamp(self, ldb, verbose=True):
        # Find PIC timestamp
        if verbose:
            print('Querying CALS for PIC abort signal, ST_ABORT_PIC.')
        return PowerConverterQuery.get_st_abort_pic_timestamp(self.circuit_type, self.circuit_name,
                                                              self.timestamp_fgc, ldb)

    def get_power_converter_earth_current(self, timestamp_fgc_ref, verbose=True):
        if verbose:
            print('Querying PM for Power Converter earth current, IEARTH.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        return PowerConverterQuery.query_earth_current(metadata_fgc, self.timestamp_fgc, timestamp_fgc_ref)

    def get_power_converter_earth_current_pcnt(self, timestamp_fgc_ref, scaling, verbose=True):
        if verbose:
            print('Querying PM for Power Converter earth current, I_EARTH_PCNT.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        return PowerConverterQuery.query_earth_current_pcnt(metadata_fgc, self.timestamp_fgc, timestamp_fgc_ref,
                                                            scaling=scaling)

    def get_power_converter_reference_current(self, verbose=True):
        if verbose:
            print('Querying PM for Power Converter reference current, STATUS.I_REF.')

        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'PC',
                                                                         'PM')
        return PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, 'STATUS.I_REF', self.timestamp_fgc)

    def get_energy_extraction_voltage(self, verbose=True):
        if verbose:
            print('Querying PM for Energy Extraction voltage, U_DUMP_RES.')

        metadata_ee = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'EE',
                                                                        'PM')
        return EnergyExtractionQuery.query_u_dump_res(metadata_ee, self.timestamp_fgc, self.circuit_name)

    def get_energy_extraction_temperature_pm(self, verbose=True):
        if verbose:
            print('Querying PM for Energy Extraction temperature, T_RES_BODY.')

        metadata_ee = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'EE',
                                                                        'PM')
        T_RES_BODY = [
            SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'EE', 'PM')[
                'T_RES_BODY_{}'.format(i)] for i in range(1, 4)]
        return EnergyExtractionQuery.query_t_res_body_pm(metadata_ee, T_RES_BODY, self.timestamp_fgc)

    def get_energy_extraction_temperature_cals(self, ldb, verbose=True):
        if verbose:
            print('Querying CALS for Energy Extraction temperature, T_RES_BODY.')

        variable_name = ["T_RES_BODY_1", "T_RES_BODY_2", "T_RES_BODY_3"]
        return EnergyExtractionQuery.query_cals(self.circuit_type, self.circuit_name, self.timestamp_fgc,
                                                variable_name, ldb)

    def get_energy_extraction_status_temperature_cals(self, ldb, verbose=True):
        if verbose:
            print('Querying CALS for Energy Extraction temperature, ST_RES_OVERTEMP.')

        variable_name = "ST_RES_OVERTEMP"
        return EnergyExtractionQuery.query_cals(self.circuit_type, self.circuit_name, self.timestamp_fgc,
                                                variable_name, ldb)

    def get_nqps_voltage(self, system, signal, ldb, prefix='', verbose=True):
        if verbose:
            print('Querying CALS for {} voltage, {}.'.format(system, signal))

        return NqpsQuery.query_nqps_voltage(self.circuit_type, self.circuit_name, self.timestamp_fgc, system=system,
                                            signal=signal, prefix=prefix, ldb=ldb)

    def get_nqps_pm_source_timestamp_df(self, verbose=True):
        if verbose:
            print('Querying PM for QPS diode voltage, U_DIODE_RB.')

        metadata_diode = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name,
                                                                           'DIODE_{}'.format(self.circuit_type), 'PM')
        source_timestamp_nqps = PmDbRequest.find_events('*', metadata_diode['system'], metadata_diode['className'],
                                                        t_start=self.timestamp_fgc, duration=[(10, 's'), (400, 's')])

        print("Total nQPS PM events on {}: {}".format(Time.to_string(self.timestamp_fgc),
                                                      len(source_timestamp_nqps)))

        if len(source_timestamp_nqps) > 0:
            source_timestamp_nqps_df = pd.DataFrame(source_timestamp_nqps).drop_duplicates(subset=[0]).sort_values(
                by=[1]).reset_index(drop=True)
            source_timestamp_nqps_df.columns = ['nQPS', 'timestamp_nqps']
            source_timestamp_nqps_df['timestamp_nqps'] = source_timestamp_nqps_df['timestamp_nqps'].astype('Int64')
        else:
            source_timestamp_nqps_df = pd.DataFrame(columns=['nQPS', 'timestamp_nqps'])
        return source_timestamp_nqps_df

    def create_report_analysis(self, source_timestamp_nqps_df, i_meas_df, verbose=True):
        if verbose:
            print('Querying PM for QPS quench detection voltage, U_QS0.')

        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'QDS',
                                                                         'PM')
        event_source_timestamp_qds = PmDbRequest.find_events('*', metadata_qds['system'], metadata_qds['className'],
                                                             t_start=self.timestamp_fgc,
                                                             duration=[(1, 's'), (1000, 's')])
        # Take only the qds events in the analyzed circuit
        event_source_timestamp_qds_filtered = list(
            filter(lambda x: SignalMetadata.get_sector_names_from_components(self.circuit_type, 'MAGNET', x[0]) in
                             self.circuit_name, event_source_timestamp_qds))

        # Create report analysis template
        report_analysis = IqdsAnalysis.create_report_analysis_template(self.circuit_type, self.circuit_name, metadata_qds,
                                                                       event_source_timestamp_qds_filtered, i_meas_df,
                                                                       self.timestamp_fgc)
        report_analysis['doneBy'] = self.author
        # nQPS - append timestamps
        report_analysis = report_analysis.merge(source_timestamp_nqps_df[['timestamp_nqps', 'nQPS']],
                                                on='nQPS', how='left').sort_values(by='timeStampPM').reset_index(
                                                drop=True)

        return report_analysis

    def get_iqps_nqps_voltage_logic(self, report_analysis, ldb, verbose=True):
        if verbose:
            print(
                'Querying PM and CALS for QPS quench detection voltage and logic signals, U_QS0, U_1, U_2, U_DIODE_RB, ST_NQD0, ST_MAGNET_OK.')

        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata(self.circuit_type, self.circuit_name, 'QDS',
                                                                         'PM')
        # ToDo: input: list of source_qds, list of source_nqs, list of timestamp_qds, list of timestamp_nqps, list of i_quench to avoid using report_analysis
        u_qds_dfs, u_nqps_dfs, st_nqps_dfs = [], [], []

        for index, row in tqdm(report_analysis.iterrows(), total=report_analysis.shape[0], desc='Querying PM'):
            source_qds, source_nqps, timestamp_qds, timestamp_nqps, i_quench = row[
                ['magnet', 'nQPS', 'timeStampPM', 'timestamp_nqps', 'qCurrent']]

            # Query
            # # iQPS
            u_qds_df = IqdsQuery.query_synchronize_pm_signal(metadata_qds,
                                                             ['U_QS0', 'U_1', 'U_2', 'ST_NQD0', 'ST_MAGNET_OK'],
                                                             timestamp_qds, source_qds)
            u_qds_df[['U_QS0', 'U_1', 'U_2']] = u_qds_df[['U_QS0', 'U_1', 'U_2']].apply(sp.signal.medfilt)
            u_qds_dfs.append(u_qds_df)
            # # nQPS
            u_nqps_df = NqdsQuery.query_nqps_pm_cals(self.circuit_type, self.circuit_name, source_nqps,
                                                     timestamp_nqps, timestamp_qds, ldb)
            u_nqps_dfs.append(u_nqps_df)

        return u_qds_dfs, u_nqps_dfs

    def get_qh_voltage_current(self, report_analysis, verbose=True):
        if verbose:
            print('Querying PM for QH voltage and current, U_HDS, I_HDS.')
        qh_sync_decay_dfs, qh_ref_sync_decay_dfs = [], []
        for index, row in tqdm(report_analysis.iterrows(), total=report_analysis.shape[0], desc='Querying PM'):
            timestamp_qds = row['timeStampPM'] + 1000000
            source_qds = row['magnet']

            # Acquire
            qh_sync_decay_df, qh_ref_sync_decay_df = QuenchHeaterQuery.get_actual_reference_discharges(
                self.circuit_type, self.circuit_name, source_qds, timestamp_qds)
            qh_sync_decay_dfs.append(qh_sync_decay_df)
            qh_ref_sync_decay_dfs.append(qh_ref_sync_decay_df)

        return qh_sync_decay_dfs, qh_ref_sync_decay_dfs

    def get_diode_leads_voltage_current(self, report_analysis, ldb, verbose=True):
        if verbose:
            print('Querying PM for diode lead resistance voltage and current, U_DIODE_RB, U_REF, IAB.I_A, I_MEAS.')

        iab_i_as, u_diode_as, u_diode_bs, u_refs = [], [], [], []
        i_circuit_syncs, u_diode_syncs = [], []
        for index, row in tqdm(report_analysis.iterrows(), total=report_analysis.shape[0], desc='Querying PM'):
            source_qds, timestamp_qds, i_quench = row[['magnet', 'timeStampPM', 'qCurrent']]

            if i_quench < 1000:
                print('The query skipped as the quench current is below 1 kA ({} A)'.format(i_quench))
            else:
                # Query
                # # PM only if a quench is within two seconds after an nqps timestamp
                timestamp_diode_fgc = DiodeLeadResistanceQuery.find_pc_pm_entries(self.circuit_type, self.circuit_name,
                                                                                  timestamp_qds)
                timestamp_diode_nqps = DiodeLeadResistanceQuery.find_diode_pm_entries(self.circuit_type,
                                                                                      self.circuit_name, source_qds,
                                                                                      timestamp_qds)
                if len(timestamp_diode_nqps) > 0:
                    is_quench_in_nqps = abs(timestamp_diode_nqps[0] - timestamp_qds) < 2e9
                else:
                    is_quench_in_nqps = False

                if is_quench_in_nqps:
                    # # # IAB.I_A
                    iab_i_as.append(DiodeLeadResistanceQuery.query_iab_i_a(self.circuit_type, self.circuit_name,
                                                                           timestamp_diode_fgc[0]))
                    # # # U_DIODE
                    u_diode_as.append(
                        DiodeLeadResistanceQuery.query_u_diode(self.circuit_type, self.circuit_name, source_qds,
                                                               timestamp_diode_nqps[0]))
                    u_diode_bs.append(
                        DiodeLeadResistanceQuery.query_u_diode(self.circuit_type, self.circuit_name, source_qds,
                                                               timestamp_diode_nqps[1]))
                    # # # U_REF
                    u_refs.append(DiodeLeadResistanceQuery.query_u_ref(self.circuit_type, self.circuit_name, source_qds,
                                                                       timestamp_diode_nqps[0]))

                # # CALS
                magnet_type, I_MEAS, U_DIODE = DiodeLeadResistanceQuery.get_magnet_type_i_meas_u_diode(source_qds,
                                                                                                       self.circuit_name)

                i_circuit_syncs.append(
                    DiodeLeadResistanceQuery.query_cals_signal(I_MEAS, timestamp_qds, magnet_type, ldb=ldb))
                u_diode_syncs.append(
                    DiodeLeadResistanceQuery.query_cals_signal(U_DIODE, timestamp_qds, magnet_type, ldb=ldb))

        return iab_i_as, u_diode_as, u_diode_bs, u_refs, i_circuit_syncs, u_diode_syncs

    def get_current_leads_source_timestamp(self, leads_type, verbose=True):
        if verbose:
            print('Querying PM for {} events.'.format(leads_type))

        return CurrentLeadsQuery.query_leads_events(self.circuit_type, self.circuit_name, self.timestamp_fgc,
                                                    leads_type=leads_type)

    def get_current_leads_voltage(self, source_leads, timestamp_leads, leads_type, verbose=True):
        if verbose:
            print('Querying PM for {} voltage, U_RES, U_HTS.'.format(leads_type))

        return CurrentLeadsQuery.query_u_leads(self.circuit_type, self.circuit_name, source_leads, timestamp_leads,
                                               leads_type=leads_type)
